#ifndef __lidar_h
#define __lidar_h

#define NEWJACOBIAN
#define CORRECTJACOBIAN
#undef MOON

/* local estimator type */
#define MCLIDAR_NONE              0
#define MCLIDAR_SPACE             1
#define MCLIDAR_FALCON            2
#define MCLIDAR_ASCOPE            3
#define MCLIDAR_POLARIZE          4
#define MCLIDAR_SIMPLE            5
#define MCLIDAR_TEST              6
#define MCLIDAR_NODDIS            7
#define MCLIDAR_PILOT             8
#define MCLIDAR_MOON              9

typedef struct qidd_struct {
  int vis;
  int ris;
  int willvis;
  int willris;
  int recalc;

  double vis_rfac;
  double vis_r;
  double vis_step;
  double vis_step2;
  double vis_dist;
  double vis_cosalpha;

  double ris_r;
  double ris_step;
} qidd_struct;

/* Initialization functions */

int set_lidar_settings (int locest, double le_abserr, sample_struct *sample);

int mc_lidar_fix_taumax (sample_struct *sample, atmosphere_struct *atmos,
			 result_struct *result, elevation_struct *elev,
			 photon_struct *p, float *refind, float r_earth,
			 int quiet);

void mc_lidar_setup_ris_mas (atmosphere_struct *atmos, sample_struct *sample);

lidar_result_field *calloc_lidar_result_field (int Nli, int Nt, int ntypes, int norders,
					       int Np, int No, int Na,
					       int std, int jacobian, int Nz);
jacobian_result_field *calloc_jacobian_result_field (int std, int Nx, int Ny, int Nz);

void mc_lidar_setup_jac_exp (sample_struct *sample, atmosphere_struct *atmos);

void calloc_photon_lidar_part (photon_struct *p, sample_struct *sample,
					     int Nz, int source);

/* finishing functions */

void destroy_photon_lidar_part (photon_struct *photon);

void free_lidar_result_field (lidar_result_field *res, int Nli, int Nt, int std);
void free_jacobian_result_field (jacobian_result_field *res);

/* functions outside photon journey */

int generate_photon_lidar_part (photon_struct *p, sample_struct *sample, atmosphere_struct *atmos, int source);
int initialize_vis_fod (sample_struct *sample, photon_struct *p);

void lidar_collect_jacobian_stddev (lidar_result_field *result_lidar, int Nt, int ili);
void abs_collect_jacobian_stddev (jacobian_result_field *result_jacobian, int is, int js);

int test_if_reached_stddev (sample_struct *sample, result_struct *result, atmosphere_struct *atmos,
				   int nphot, int quiet);

/* travel_tau functions */

void mc_lidar_prepare_travel_tau (sample_struct *sample, photon_struct *p,
				  int *moving_towards_detector);


int mc_lidar_prepare_travel_tau_step (sample_struct *sample, photon_struct *p,
				      qidd_struct *qidd, atmosphere_struct *atmos,
				      double dtau,
				      int moving_towards_detector);

int mc_lidar_travel_tau_action (photon_struct *p, double *step, qidd_struct *qidd);

double ksca_for_tausca (photon_struct *p, double *step, double *ksca,
			qidd_struct *qidd);

void mc_lidar_travel_tau_addition (atmosphere_struct *atmos, sample_struct *sample,
				   photon_struct *p,
				   double *step, qidd_struct *qidd, double *ksca);

void mc_lidar_travel_special_cases (sample_struct *sample, photon_struct *p,
				    qidd_struct *qidd, double *ksca, double dtau);

/* local estimator functions */

int mc_lidar_locest (result_struct *result, sample_struct *sample,
		     atmosphere_struct *atmos,
		     photon_struct *photon, photon_struct *p,
		     int itl, int LoWeRR_crit);

int lidar_detector_kernel (sample_struct *sample, photon_struct *p);


/* helper functions */

int calc_locest_connection (photon_struct *p, sample_struct *sample,
			    double *x_end, locest_struct *lest_out);

int derive_cphi (double *dx1, double *dx2, double stheta, double *cphi);

/* scattering functions */

int calc_distance_to_cone (double c_th, double s_th, double cphi2,
			   double y_sc_cc, double z_sc_cc, double l_sc_cc_inv,
			   double t_det,
			   locest_struct *lest);

/* non-lidary functions */

int intersection3D_spherical (photon_struct *p,
			      atmosphere_struct *atmos,
			      int *first,
			      double *step);

int atmos_coord_spherical3D (photon_struct *p, atmosphere_struct *atmos,
			     int *ic, int *jc, int *kc);

int setup_spherical3D_atmos (atmosphere_struct *atmos);

int generate_photon_spherical3D (sample_struct *sample, photon_struct *p,
				 atmosphere_struct *atmos,
				 double sza, double phi);

#endif
